
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2015 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbBoxConvert.h"
#include "dbCell.h"

namespace db
{

template <bool AllowEmpty>
db::Box
box_convert <db::CellInst, AllowEmpty>::operator() (const db::CellInst &t) const
{
  if (m_layer >= 0) {
    return t.bbox (*mp_layout, (unsigned int) m_layer);
  } else if (AllowEmpty) {
    return t.bbox (*mp_layout);
  } else {
    db::Box box = t.bbox (*mp_layout);
    if (box.empty ()) {
      return db::Box (db::Point (0, 0), db::Point (0, 0));
    } else {
      return box;
    }
  }
}

template <bool AllowEmpty>
db::Box
box_convert <db::Cell, AllowEmpty>::operator() (const db::Cell &c) const
{
  if (m_layer >= 0) {
    return c.bbox ((unsigned int) m_layer);
  } else if (AllowEmpty) {
    return c.bbox ();
  } else {
    if (c.bbox ().empty ()) {
      return db::Box (db::Point (0, 0), db::Point (0, 0));
    } else {
      return c.bbox ();
    }
  }
}

template struct box_convert <db::CellInst, true>;
template struct box_convert <db::CellInst, false>;
template struct box_convert <db::Cell, true>;
template struct box_convert <db::Cell, false>;

}

