
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2015 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "gsi.h"
#include "gsiDecl.h"

namespace gsi
{

// --------------------------------------------------------------------------------
//  Implementation of ArgType

std::string
ArgType::to_string () const
{
  std::string s;
  if (is_cref || is_cptr) {
    s = "const ";
  }
  switch (type) {
  case T_void:
    s += "void"; break;
  case T_void_ptr:
    s += "void *"; break;
  case T_bool:
    s += "bool"; break;
  case T_char:
    s += "char"; break;
  case T_schar:
    s += "signed char"; break;
  case T_uchar:
    s += "unsigned char"; break;
  case T_short:
    s += "short"; break;
  case T_ushort:
    s += "unsigned short"; break;
  case T_int:
    s += "int"; break;
  case T_uint:
    s += "unsigned int"; break;
  case T_long:
    s += "long"; break;
  case T_ulong:
    s += "unsigned long"; break;
  case T_longlong:
    s += "long long"; break;
  case T_ulonglong:
    s += "unsigned long long"; break;
  case T_double:
    s += "double"; break;
  case T_float:
    s += "float"; break;
  case T_string_ccptr:
    s += "const char *"; break;
  case T_string:
    s += "string"; break;
  case T_qstring:
    s += "QString"; break;
  case T_var:
    s += "variant"; break;
  case T_object_new:
    s += "new " + cls->name (); break;
  case T_object:
    s += cls->name (); break;
  case T_vector:
    if (inner) {
      s += inner->to_string ();
    } 
    s += "[]";
    break;
  }
  if (is_cref || is_ref) {
    s += " &";
  } else if (is_cptr || is_ptr) {
    s += " *";
  }
  return s;
}

ArgType::ArgType ()
  : type (T_void), inner (0), is_ref (false), is_ptr (false), is_cref (false), is_cptr (false), is_iter (false), cls (0), size (0)
{ }

ArgType::~ArgType ()
{
  if (inner) {
    delete inner;
    inner = 0;
  }
}

ArgType::ArgType (const ArgType &other)
  : type (T_void), inner (0), is_ref (false), is_ptr (false), is_cref (false), is_cptr (false), is_iter (false), cls (0), size (0)
{
  operator= (other);
}

ArgType &
ArgType::operator= (const ArgType &other)
{
  if (this != &other) {

    type = other.type;
    is_ref = other.is_ref;
    is_cref = other.is_cref;
    is_ptr = other.is_ptr;
    is_cptr = other.is_cptr;
    is_iter = other.is_iter;
    cls = other.cls;
    size = other.size;

    if (inner) {
      delete inner;
      inner = 0;
    }

    if (other.inner) {
      inner = new ArgType (*other.inner);
    }

  }

  return *this;
}

}

